# LiPo-Mointor-GUI
# by Johannes Kinzig

# Version 1.0 unstable
#       - because of thread module: function (endless loop) is called in seperated thread and cannot be silently terminated/killed


# Platform dependent (OS X):
#       - listing the serial ports

# requires following packages (only packages which are not part of the main contribution are listed)
#       - pyserial

import serial
import Tkinter
import ttk
import subprocess
import tkMessageBox
import time
import thread


#********************************************************************#
#                          Main program                              #
#********************************************************************#
readData = [10]

#*************************************************#
#               Serial ports                      #
#*************************************************#

# list all available serial ports: ask System via ls /dev/tty.*
proc = subprocess.Popen("ls /dev/tty.*", shell=True, stdout = subprocess.PIPE)
output = proc.communicate()[0]

#now list the information into an array to access them independently
seriallist = output.split()
print seriallist


# Connect to the chosen serial device!
def ConnectToSerialDevice():
	global serialDevice
	chosenDevice = serialCombobox.get()
	print chosenDevice
	try:
		serialDevice = serial.Serial(chosenDevice)
	except:
		errormessage = "Sorry, I could not connect to the device:\n" + chosenDevice
		print errormessage
		tkMessageBox.showerror("Error connecting to device...", errormessage)
	
# Disconnect from chosen serial device!
def DisconnectFromSerialDevice():
	serialDevice.close()


#*************************************************#
#               Read in Data                      #
#*************************************************#

def GetDataFromSerialDevice():
        while True:
                try:
                        serialDevice.write("GET_CAPACITY")

                        del readData[:]
                        for i in range(0, 10):
                                readData.append(serialDevice.readline().rstrip())

                        print readData
                        LabelLiPo_Percent.set(readData[1])
                        LabelCELL_1.set(readData[3])
                        LabelCELL_2.set(readData[5])
                        LabelUSED_CAPACITY.set(readData[7])
                        LabelCURRENT_CURRENT.set(readData[9])

                        # update progressbar with USED_CAPACITY
                        ProgbarUSED_CAPACITY["value"] = readData[7]
                        ProgbarUSED_CAPACITY.update()

                        # update progressbar with CURRENT_CURRENT
                        ProgbarCURRENT_CURRENT["value"] = readData[9]
                        ProgbarCURRENT_CURRENT.update()
                except:
                        break

#********************************************************************#
#                          GUI instances                             #
#********************************************************************#

# Window instance
root = Tkinter.Tk()
root.title("LiPo Monitor 1.0")

mainwindow = ttk.Frame(root)
mainwindow.grid()
root.resizable(width=False, height=False)

# generate label for SerialDevice
ttk.Label(mainwindow, text = "Serial Device:").grid(row = 0, column = 0)

# generate Button "Connect"
ttk.Button(mainwindow, text = "Connect", command = ConnectToSerialDevice).grid(row = 0, column = 2)

# generate Button "Disconnect"
ttk.Button(mainwindow, text = "Disconnect", command = DisconnectFromSerialDevice).grid(row = 0, column = 3)

# generate Combo-box for choosing the serial device
serialCombobox = Tkinter.StringVar()
serialCombobox.set('Available Serial Ports')
ttk.Combobox(mainwindow, values = seriallist, textvariable=serialCombobox, state = "readonly", width = 30).grid(row = 0, column = 1)

# generate Button "GetData"
ttk.Button(mainwindow, text = "GetData", command = lambda: thread.start_new_thread(GetDataFromSerialDevice, ())).grid(row = 1, column = 2)

################################# Data Labels for derived serial data ######################################

# generate Label for LiPo_Percent description:
ttk.Label(mainwindow, text = "Percentage:").grid(row = 1, column = 0)

# generate Label for LiPo_Percent data:
LabelLiPo_Percent = Tkinter.StringVar()
LabelLiPo_Percent.set('n. a.')
ttk.Label(mainwindow, textvariable = LabelLiPo_Percent).grid(row = 1, column = 1)

# generate Label for CELL_1 description:
ttk.Label(mainwindow, text = "Cell 1 (V):").grid(row = 2, column = 0)

# generate Label for CELL_1 data:
LabelCELL_1 = Tkinter.StringVar()
LabelCELL_1.set('n. a.')
ttk.Label(mainwindow, textvariable = LabelCELL_1).grid(row = 2, column = 1)

# generate Label for CELL_2 description:
ttk.Label(mainwindow, text = "Cell 2 (V):").grid(row = 3, column = 0)

# generate Label for CELL_2 data:
LabelCELL_2 = Tkinter.StringVar()
LabelCELL_2.set('n. a.')
ttk.Label(mainwindow, textvariable = LabelCELL_2).grid(row = 3, column = 1)

# generate Label for USED_CAPACITY description:
ttk.Label(mainwindow, text = "Used capacity (mAh):").grid(row = 4, column = 0)

# generate Label for USED_CAPACITY data:
LabelUSED_CAPACITY = Tkinter.StringVar()
LabelUSED_CAPACITY.set('n. a.')
ttk.Label(mainwindow, textvariable = LabelUSED_CAPACITY).grid(row = 4, column = 1)

# generate Label for CURRENT_CURRENT description:
ttk.Label(mainwindow, text = "Current current (mA):").grid(row = 5, column = 0)

# generate Label for CURRENT_CURRENT data:
LabelCURRENT_CURRENT = Tkinter.StringVar()
LabelCURRENT_CURRENT.set('n. a.')
ttk.Label(mainwindow, textvariable = LabelCURRENT_CURRENT).grid(row = 5, column = 1)

# generate Progressbar for USED_CAPACITY
ProgbarUSED_CAPACITY = ttk.Progressbar(mainwindow)
ProgbarUSED_CAPACITY.grid(row = 4, column = 2, padx = 10, pady = 10)
ProgbarUSED_CAPACITY["maximum"] = 850
ProgbarUSED_CAPACITY["value"] = 0

# generate Progressbar for CURRENT_CURRENT
ProgbarCURRENT_CURRENT = ttk.Progressbar(mainwindow)
ProgbarCURRENT_CURRENT.grid(row = 5, column = 2, padx = 10, pady = 10)
ProgbarCURRENT_CURRENT["maximum"] = 5000
ProgbarCURRENT_CURRENT["value"] = 0

root.mainloop()
