// by Johannes Kinzig
// Version 1.0
// Records capacity taken out of battery and warns when max possible capacity is taken out of it
// Send "GET_CAPACITY" to the board via UART-TTL (RS485 is also possible) and the board answers with the cells voltage, state of charge in percent and the current current


/**********************************************/
/*          Change these as you need          */
/**********************************************/
float LIPO_CAPACITY = 850; // mAh


/**********************************************/
/*           BEGIN CLASS DEFINITION           */   
/*               CLASS LED_BAR                */
/**********************************************/
class LED_BAR {
  public:
    LED_BAR(int led1, int led2, int led3, int led4, int led5, int led6);
    void flash(int sleep);
    void lightup(int percent);
  private:
  int _led1, _led2, _led3, _led4, _led5, _led6;
};

LED_BAR::LED_BAR(int led1, int led2, int led3, int led4, int led5, int led6) {
  
  _led1 = led1;
  _led2 = led2;
  _led3 = led3;
  _led4 = led4;
  _led5 = led5;
  _led6 = led6;
  
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(led3, OUTPUT);
  pinMode(led4, OUTPUT);
  pinMode(led5, OUTPUT);
  pinMode(led6, OUTPUT);
}

void LED_BAR::flash(int sleep) {
  digitalWrite(_led1, HIGH);
  digitalWrite(_led2, HIGH);
  digitalWrite(_led3, HIGH);
  digitalWrite(_led4, HIGH);
  digitalWrite(_led5, HIGH);
  digitalWrite(_led6, HIGH);
  delay(sleep);
  
  digitalWrite(_led1, LOW);
  digitalWrite(_led2, LOW);
  digitalWrite(_led3, LOW);
  digitalWrite(_led4, LOW);
  digitalWrite(_led5, LOW);
  digitalWrite(_led6, LOW);
  delay(sleep);
}

void LED_BAR::lightup(int percent) {
  int led_percent = 100 / 6;
  
  if (0 < percent && percent < led_percent) {
    
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, LOW);
    digitalWrite(_led3, LOW);
    digitalWrite(_led4, LOW);
    digitalWrite(_led5, LOW);
    digitalWrite(_led6, LOW);
  }
  else if (percent < 2*led_percent) {
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, HIGH);
    digitalWrite(_led3, LOW);
    digitalWrite(_led4, LOW);
    digitalWrite(_led5, LOW);
    digitalWrite(_led6, LOW);
  }
  else if (percent < 3*led_percent) {
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, HIGH);
    digitalWrite(_led3, HIGH);
    digitalWrite(_led4, LOW);
    digitalWrite(_led5, LOW);
    digitalWrite(_led6, LOW);
  }
  else if (percent < 4*led_percent) {
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, HIGH);
    digitalWrite(_led3, HIGH);
    digitalWrite(_led4, HIGH);
    digitalWrite(_led5, LOW);
    digitalWrite(_led6, LOW);
  }
  else if (percent < 5*led_percent) {
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, HIGH);
    digitalWrite(_led3, HIGH);
    digitalWrite(_led4, HIGH);
    digitalWrite(_led5, HIGH);
    digitalWrite(_led6, LOW);
  }
  else if (percent < 101) {
    digitalWrite(_led1, HIGH);
    digitalWrite(_led2, HIGH);
    digitalWrite(_led3, HIGH);
    digitalWrite(_led4, HIGH);
    digitalWrite(_led5, HIGH);
    digitalWrite(_led6, HIGH);
  }
}
/**********************************************/
/*             END CLASS DEFINITION           */   
/*               CLASS LED_BAR                */
/**********************************************/

LED_BAR leds(5, 6, 7, 8, 9, 10);

/**********************************************/
/*           BEGIN Variable DEFINITION        */   
/*                                            */
/**********************************************/

float CELL_1;
float CELL_2;
int value_cell_1;
int value_cell_2;
int LIPO_PERCENT;
int value_current_current;
float USED_CAPACITY;
float CURRENT_CURRENT;
char inChar;
String inString = "";


/**********************************************/
/*           END Variable DEFINITION          */   
/*                                            */
/**********************************************/

// the setup routine runs once when you press reset:
void setup() {
  pinMode(A1, INPUT); // CELL 1 to ground
  pinMode(A2, INPUT); // CELL 2 to ground
  pinMode(11, OUTPUT);
  pinMode(2, OUTPUT);
  digitalWrite(2, LOW);
  Serial.begin(9600);
  inString.reserve(15); // reserve 15 bytes
}


void loop() {
  // Voltage CELL_1
  value_cell_1 = analogRead(A1);
  CELL_1 = value_cell_1 * (5.0 / 1024.0);
  
  // Voltage CELL_2
  value_cell_2 = analogRead(A2);
  CELL_2 = value_cell_2 * (5.0 / 1024.0);
  CELL_2 = (CELL_2 * 2) - CELL_1;
  
  
  // Test if cell voltage is low
  if (CELL_1 > 3.3 && CELL_2 > 3.3) {
    analogWrite(11, 0);
    
    if (USED_CAPACITY < LIPO_CAPACITY) {
      analogWrite(11, 0);
    }
    else {
      bat_alert();
    }
  }
  else {
    bat_alert();
  }
  
  // Current current; Sensor: Max 5A EQ 5V
  value_current_current = analogRead(A0);
  CURRENT_CURRENT = (value_current_current * (5000.0 / 1024.0)); // convert to mV (float)
  
  USED_CAPACITY = USED_CAPACITY + (CURRENT_CURRENT / 3600); // converts mAs to mAh (float)
  
  LIPO_PERCENT = 100 - (100 / LIPO_CAPACITY * USED_CAPACITY);
  
  leds.lightup(LIPO_PERCENT);
  
  delay(950);
}

void bat_alert() {
  // battery is empty or cell voltage is low
  analogWrite(11, 200); // BUZZER
  leds.flash(100);
  analogWrite(11, 0); // BUZZER
  leds.flash(100);
}

void serialEvent() {
  // data is transmitted via the serial interface or RS485
  while (Serial.available() > 0) {
    char inChar = (char)Serial.read();
    inString = inString + inChar;
    
    //Serial.println(inString);
    
    if (inString == "GET_CAPACITY") {
      
      digitalWrite(2, HIGH); //ENAB_RS485 = true
      delay(50); // RS485 transceiver should have time to wake up
      
      Serial.println("LIPO_PERCENT:");
      Serial.println(LIPO_PERCENT);
  
      Serial.println("CELL_1 (V):");
      Serial.println(CELL_1);
    
      Serial.println("CELL_2 (V):");
      Serial.println(CELL_2);

      Serial.println("USED_CAPACITY:");
      Serial.println(USED_CAPACITY);
  
      Serial.println("CURRENT_CURRENT:");
      Serial.println(CURRENT_CURRENT);
      
      delay(50); // RS485 transceiver should have time to go to sleep
      digitalWrite(2, LOW); //ENAB_RS485 = false
  }
 }
 inString = ""; // clear String which is compared later
}
